import 'dart:async';
import 'dart:convert';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:sharing/common/values/colors.dart';
import 'package:sharing/remote/constants.dart';

final SharedPrefsHelper prefsHelper = SharedPrefsHelper();

class SharedPrefsHelper {
  static final instance = SharedPrefsHelper._internal();
  static SharedPreferences? _prefs;

  factory SharedPrefsHelper() => instance;

  SharedPrefsHelper._internal();

  Future<void> initialize() async {
    _prefs = await SharedPreferences.getInstance();
  }

  setThemeChange(bool isChange) =>
      _prefs!.setBool(Constants.THEME_CHANGE_CACHE, isChange);

  getThemeChange() => _prefs?.getBool(Constants.THEME_CHANGE_CACHE) ?? false;

  setDomainUrl(String value) =>
      _prefs!.setString(Constants.DOMAIN_URL_CACHE, value);

  getDomainUrl() => _prefs?.getString(Constants.DOMAIN_URL_CACHE);

  setLogo(String value) => _prefs!.setString(Constants.LOGO_CACHE, value);

  getLogo() => _prefs?.getString(Constants.LOGO_CACHE);

  setImageUrl(String value) =>
      _prefs!.setString(Constants.IMAGE_URL_CACHE, value);

  getImageUrl() => _prefs?.getString(Constants.IMAGE_URL_CACHE);

  setPrimaryColor(String value) =>
      _prefs?.setString(Constants.PRIMARY_COLOR_CACHE, value);

  getPrimaryColor() {
    final color = _prefs!.getString(Constants.PRIMARY_COLOR_CACHE);
    if (color?.isNotEmpty == true) {
      return color;
    } else {
      return SharingColors.primaryColor;
    }
  }

  setPrimarySecondColor(String value) =>
      _prefs?.setString(Constants.PRIMARY_SECOND_COLOR_CACHE, value);

  getPrimarySecondColor() {
    final color = _prefs?.getString(Constants.PRIMARY_SECOND_COLOR_CACHE);
    if (color?.isNotEmpty == true) {
      return color;
    } else {
      SharingColors.primary2Color;
    }
  }

  setSecondColor(String value) =>
      _prefs?.setString(Constants.SECOND_COLOR_CACHE, value);

  getSecondColor() {
    final color = _prefs?.getString(Constants.SECOND_COLOR_CACHE);
    if (color?.isNotEmpty == true) {
      return color;
    } else {
      SharingColors.secondaryColor;
    }
  }

  setLocale(String value) =>
      _prefs?.setString(Constants.LOCALE_CACHE, value);

  getLocale() => _prefs?.getString(Constants.LOCALE_CACHE) ?? "vi";

  setButtonColor(String value) =>
      _prefs?.setString(Constants.BUTTON_COLOR_CACHE, value) ;

  getButtonColor() => _prefs?.getString(Constants.BUTTON_COLOR_CACHE) ?? SharingColors.primary2Color;

  setFontName(String? value) =>
      _prefs?.setString(Constants.FONT_CACHE, value ?? "Roboto");

  getFontName() => _prefs?.getString(Constants.FONT_CACHE) ?? "Roboto";

  setBeginColor(String value) =>
      _prefs?.setString(Constants.BEGIN_COLOR_CACHE, value);

  List<double>? getBeginColor() {
    final data = _prefs?.getString(Constants.BEGIN_COLOR_CACHE);
    if (data != null) {
      return json.decode(data).cast<double>();
    } else {
      return null;
    }
  }

  setEndColor(String value) =>
      _prefs?.setString(Constants.END_COLOR_CACHE, value);

  List<double>? getEndColor() {
    final data = _prefs?.getString(Constants.END_COLOR_CACHE);
    if (data != null) {
      return json.decode(data).cast<double>();
    } else {
      return null;
    }
  }

  clearData() {
    _prefs?.clear();
  }
}
