import '../../common/values/colors.dart';
import '../../common/values/sizes.dart';
import '../../common/values/styles.dart';
import 'package:easy_localization/src/public_ext.dart';
import 'package:flutter/material.dart';

import '../custom/custom_text_form_field.dart';


// ignore: must_be_immutable
class CustomDialogNote extends StatefulWidget {
  final String positiveBtnText;
  final Function(String)? positiveBtnPressed;
  String? note;

  CustomDialogNote(
      {required this.positiveBtnText,
      required this.positiveBtnPressed,
      this.note});

  @override
  State<CustomDialogNote> createState() => _CustomDialogNoteState();
}

class _CustomDialogNoteState extends State<CustomDialogNote> {
  final TextEditingController _ctrNote = TextEditingController();
  final FocusNode _focusNote = FocusNode();

  @override
  void initState() {
    super.initState();
    _ctrNote.text = widget.note ?? "";
  }

  @override
  Widget build(BuildContext context) {
    return Dialog(
      elevation: 0,
      backgroundColor: Colors.transparent,
      child: _buildDialogContent(context),
    );
  }

  Widget _buildDialogContent(BuildContext context) {
    return WillPopScope(
      onWillPop: () async => true,
      child: Container(
        margin: EdgeInsets.only(top: 40),
        decoration: BoxDecoration(
            color: SharingColors.white, borderRadius: BorderRadius.circular(8)),
        child: Column(
          mainAxisSize: MainAxisSize.min,
          crossAxisAlignment: CrossAxisAlignment.start,
          children: <Widget>[
            Container(
              padding: EdgeInsets.symmetric(horizontal: 12, vertical: 30),
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.center,
                children: [
                  CustomTextFormField(
                      prefixIcon: Icons.edit,
                      hasPrefixIcon: false,
                      controller: _ctrNote,
                      focusNode: _focusNote,
                      onChange: (value) {
                        setState(() {
                          widget.note = value;
                        });
                      },
                      maxLenght: 300,
                      maxLine: 1,
                      textInputType: TextInputType.multiline,
                      labelText: "text_note".tr()),
                ],
              ),
            ),
            Container(
              decoration: BoxDecoration(
                  color: Theme.of(context).primaryColor,
                  borderRadius: BorderRadius.only(
                      bottomLeft: Radius.circular(8),
                      bottomRight: Radius.circular(8))),
              width: Sizes.width(context),
              height: 42,
              child: TextButton(
                child: Text(
                  widget.positiveBtnText,
                  style: Styles.customTextMediumStyle(color: Colors.white),
                ),
                onPressed: () {
                  widget.positiveBtnPressed!(widget.note ?? "");
                },
              ),
            ),
          ],
        ),
      ),
    );
  }
}
